﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.UltraWinGrid;

namespace TabSample
{
    public partial class Form1 : Form
    {
        private int CellToCopy { get; set; }

        public Form1()
        {
            InitializeComponent();

            this.ultraGrid1.DisplayLayout.Override.AllowAddNew = AllowAddNew.No;
            this.ultraGrid1.DisplayLayout.Override.AllowDelete = Infragistics.Win.DefaultableBoolean.False;
            this.ultraGrid1.DisplayLayout.Override.CellClickAction = CellClickAction.RowSelect;
            this.ultraGrid1.DisplayLayout.Override.AllowUpdate = Infragistics.Win.DefaultableBoolean.False;

            DataTable dt = new DataTable();
            dt.Columns.Add("Col1");
            dt.Columns.Add("Col2");
            dt.Columns.Add("Col3");

            DataRow dr = dt.NewRow();
            dr["Col1"] = "Col1Row1";
            dr["Col2"] = "Col2Row1";
            dr["Col3"] = "Col3Row1";
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr["Col1"] = "Col1Row2";
            dr["Col2"] = "Col2Row2";
            dr["Col3"] = "Col3Row2";
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr["Col1"] = "Col1Row3";
            dr["Col2"] = "Col2Row3";
            dr["Col3"] = "Col3Row3";
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr["Col1"] = "Col1Row4";
            dr["Col2"] = "Col2Row4";
            dr["Col3"] = "Col3Row4";
            dt.Rows.Add(dr);

            this.ultraGrid1.DataSource = dt;
            this.ultraGrid1.DataBind();
        }

        private void ultraGrid1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.Shift && e.KeyCode == Keys.C)
                this.CopyText(false);
            else if (e.Control && e.KeyCode == Keys.C)
                this.CopyText(true);
            else if (e.Shift && e.KeyCode == Keys.Tab)
                this.CellToCopy = HandleTab(this.CellToCopy, false, this.ultraGrid1, e);
            else if (e.KeyCode == Keys.Tab)
                this.CellToCopy = HandleTab(this.CellToCopy, true, this.ultraGrid1, e);
        }

        /// <summary>
        /// Copies cell values to clipboard
        /// </summary>
        /// <param name="copyJustActive">indicates if just the value from the active row should be copied</param>
        private void CopyText(bool copyJustActive)
        {
            if (this.CellToCopy > -1)
            {
                StringBuilder copy = new StringBuilder();

                if (copyJustActive)
                {
                    if (this.ultraGrid1.ActiveRow != null)
                        copy.AppendLine(this.ultraGrid1.ActiveRow.Cells[this.CellToCopy].Text);
                }
                else
                {
                    for (int i = 0; i < this.ultraGrid1.Selected.Rows.Count; i++)
                    {
                        if (i == this.ultraGrid1.Selected.Rows.Count - 1)
                            copy.Append(this.ultraGrid1.Selected.Rows[i].Cells[this.CellToCopy].Text);
                        else
                            copy.AppendLine(this.ultraGrid1.Selected.Rows[i].Cells[this.CellToCopy].Text);
                    }
                }

                if (!string.IsNullOrEmpty(copy.ToString()))
                    Clipboard.SetText(copy.ToString());
            }
        }

        /// <summary>
        /// Handles the tab key. Finds the next visible cell index for copy and determines whether or not
        /// to continue processing the tab request or flag it as handled if staying within the same row.
        /// </summary>
        /// <param name="cellIndex">Index of the starting cell.</param>
        /// <param name="moveForward">moves to next index when true, previous when false.</param>
        /// <param name="grid">The grid to traverse.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
        /// <returns>index of next visible cell</returns>
        public int HandleTab(int cellIndex, bool moveForward, UltraGrid grid, KeyEventArgs e)
        {
            if (cellIndex > -1)
            {
                int startCellIndex = cellIndex;

                bool foundNextCell = false;
                bool nextRow = false;

                if (moveForward)
                {
                    cellIndex++;

                    while (!foundNextCell)
                    {
                        if (cellIndex >= grid.DisplayLayout.Bands[0].Columns.Count)
                        {
                            cellIndex = 0;
                            nextRow = true;
                        }

                        if (!grid.DisplayLayout.Bands[0].Columns[cellIndex].Hidden)
                            foundNextCell = true;
                        else
                            cellIndex++;

                        if (startCellIndex == cellIndex)
                            foundNextCell = true;
                    }
                }
                else
                {
                    cellIndex--;

                    while (!foundNextCell)
                    {
                        if (cellIndex < 0)
                        {
                            cellIndex = grid.DisplayLayout.Bands[0].Columns.Count - 1;
                            nextRow = true;
                        }

                        if (!grid.DisplayLayout.Bands[0].Columns[cellIndex].Hidden)
                            foundNextCell = true;
                        else
                            cellIndex--;

                        if (startCellIndex == cellIndex)
                            foundNextCell = true;
                    }
                }

                e.Handled = true;

                if (nextRow)
                {
                    if (moveForward)
                        grid.PerformAction(UltraGridAction.BelowRow);
                    else
                        grid.PerformAction(UltraGridAction.AboveRow);
                }
            }

            return cellIndex;
        }
    }
}
